package com.web.controller.notify;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageInfo;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.entity.HighGroupCode;
import com.hai.entity.HighGroupOrder;
import com.hai.entity.HighOrder;
import com.hai.group.GroupCodeService;
import com.hai.model.ResponseData;
import com.hai.service.HighOrderService;
import com.hai.service.impl.OutRechargeOrderServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

@Controller
@RequestMapping(value = "/groupOrder")
@Api(value = "团购订单")
public class GroupOrderController {
    private static final Logger log = LoggerFactory.getLogger(GroupOrderController.class);


    @Resource
    private GroupCodeService groupCodeService;

    @Resource
    private HighOrderService highOrderService;


    @RequestMapping(value = "/gorOrderNotify", method = RequestMethod.GET)
    @ApiOperation(value = "团购订单回调")
    @ResponseBody
    public ResponseData gorOrderNotify(@RequestParam(name = "groupCode", required = false) String groupCode,
                                       @RequestParam(name = "storeId", required = false) String storeId) {
        try {



            log.info("============回调任务Start=============");
            log.info("团购订单-回调参数: " + groupCode);

            Map<String , Object> map = new HashMap<>();
            map.put("groupCode" , groupCode);
            map.put("storeId" , storeId);

            HighGroupCode highGroupCode = groupCodeService.findGroupCode(map);


            if (highGroupCode == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到相关核销券！");
            }


            HighOrder highOrder = highOrderService.getOrderById(highGroupCode.getOrderId());

            HighGroupOrder highGroupOrder = groupCodeService.findGroupOrder(highOrder.getOrderNo());

            if (highGroupCode.getStatus() != 1 || highOrder.getOrderStatus() == 3) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "已核销不可再次核销！");
            }

            highGroupCode.setStatus(2);
            highGroupCode.setUpdateTime(new Date());
            groupCodeService.updateGroupCode(highGroupCode);


            highGroupOrder.setStatus(3);
            highGroupOrder.setUpdateTime(new Date());
            groupCodeService.updateGroupOrder(highGroupOrder);

            highOrder.setOrderStatus(3);
            highOrder.getHighChildOrderList().get(0).setChildOrderStatus(3);
            highOrderService.updateOrder(highOrder);

            log.info("============回调任务End=============");
            return ResponseMsgUtil.success("核销成功！");
        } catch (Exception e) {
            log.error("WechatPayController --> wechatNotify() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

}
