package com.web.controller.notify;

import com.alibaba.fastjson.JSONObject;
import com.hai.entity.HighOrder;
import com.hai.order.service.OrderPaySuccessService;
import com.hai.order.service.OrderService;
import com.hai.order.type.OrderPayType;
import com.hai.order.type.OrderStatus;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.PrintWriter;
import java.math.BigDecimal;
import java.util.HashMap;
import java.util.Map;

@Controller
@RequestMapping(value = "/huiPayNotify")
@Api(value = "惠支付业务通知")
public class HuiPayNotifyController {

    private static final Logger log = LoggerFactory.getLogger(HuiPayNotifyController.class);

    @Resource
    private OrderService orderService;

    @Resource
    private OrderPaySuccessService paySuccessService;


    @RequestMapping(value = "/huiPayCallback", method = RequestMethod.POST)
    @ApiOperation(value = "惠支付回调")
    @ResponseBody
    public void huiPayCallback(@RequestBody String reqBodyStr,  HttpServletResponse response) {
        try {
            log.info("===============惠支付回调start==================");
            JSONObject dataObject = JSONObject.parseObject(reqBodyStr, JSONObject.class);
            // 处理业务
            log.info("开始处理业务");
            log.info("惠支付-回调参数: " + dataObject);

            // 查询交易订单
            HighOrder order = orderService.getOrderDetailByNo(dataObject.getString("outTradeNo"));
            if (order != null && order.getOrderStatus().equals(OrderStatus.STATUS1.getNumber())) {
                OrderPayType orderPayType = null;
                if (dataObject.getString("payMode").equals("WECHAT")) {
                    orderPayType = OrderPayType.PAY_TYPE2;
                } else if (dataObject.getString("payMode").equals("ALIPAY")) {
                    orderPayType = OrderPayType.PAY_TYPE1;
                } else if (dataObject.getString("payMode").equals("UQRCODEPAY")) {
                    orderPayType = OrderPayType.PAY_TYPE5;
                }

                Map<String,Object> other = new HashMap<>();
                other.put("payChannelOrderNo", dataObject.getString("platformTradeNo"));

                // 处理订单业务
                paySuccessService.orderPaySuccessHandle(
                        dataObject.getString("outTradeNo"),
                        orderPayType,
                        dataObject.getString("accTradeNo"),
                        dataObject.getBigDecimal("tradeAmount"),
                        null,
                        other);
            }
            log.info("处理业务完成");
            log.info("============回调任务End=============");


            response.setCharacterEncoding("UTF-8");
            response.setContentType("text/html;charset=utf-8");
            PrintWriter writer= response.getWriter();

            JSONObject postJson = new JSONObject();
            postJson.put("code" ,"SUCCESS");
            postJson.put("message" ,"执行成功");
            writer.write(postJson.toJSONString());
            writer.flush();
            writer.close();

        } catch (Exception e) {
            log.error("订单处理异常", e);
        } finally {
            log.info("===============微信支付回调end==================");
        }
    }

}
