package com.hai.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.UserCenter;
import com.hai.dao.HighUserCardMapper;
import com.hai.entity.*;
import com.hai.enum_type.OilCardStatusEnum;
import com.hai.enum_type.UserCardType;
import com.hai.model.HighUserModel;
import com.hai.openApi.config.BlxConfig;
import com.hai.service.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Service("highUserCardService")
public class HighUserCardServiceImpl implements HighUserCardService {

    @Resource
    private HighUserCardMapper highUserCardMapper;

    @Resource
    private HighUserCouponService highUserCouponService;

    @Resource
    private HighCouponCodeOtherService couponCodeOtherService;

    @Resource
    private BlxConfig blxConfig;

    @Resource
    private HighOilCardService oilCardService;

    @Resource
    private HighFleetOilCardService fleetOilCardService;

    @Resource
    private HighCouponService highCouponService;

    @Resource
    private HighUserService highUserService;

    @Override
    public void editCard(HighUserCard highUserCard) {
        if(highUserCard.getId() == null){
            highUserCard.setCreateTime(new Date());
            highUserCard.setUpdateTime(new Date());
            highUserCard.setStatus(1);
            highUserCardMapper.insert(highUserCard);
        } else {
            highUserCard.setUpdateTime(new Date());
            highUserCardMapper.updateByPrimaryKey(highUserCard);
        }
    }

    @Override
    @Transactional(propagation= Propagation.REQUIRED)
    public void bindCard(Integer type, String cardNo,Long userId) {
        HighUserCard userCard = new HighUserCard();
        userCard.setType(type);
        userCard.setUserId(userId);
        userCard.setCardNo(cardNo);
        editCard(userCard);

        if (type.equals(UserCardType.type2.getType())) {
            // 查询卡号
            HighOilCard card = oilCardService.getOilCardByCardNo(cardNo);
            if (card == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "油卡卡号错误");
            }
            if (!card.getStatus().equals(OilCardStatusEnum.status1.getStatus())) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "油卡状态异常");
            }
            if (card.getBindStatus() == 1) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "油卡已被绑定，请勿重复绑定");
            }
            card.setBindTime(new Date());
            card.setBindStatus(1);
            oilCardService.editData(card);

        } else if (type.equals(UserCardType.type3.getType())) {
            // 查询卡号
            HighFleetOilCard card = fleetOilCardService.getFleetOilCardByCardNo(cardNo);
            if (card == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "车队油卡卡号错误");
            }
            if (!card.getStatus().equals(OilCardStatusEnum.status1.getStatus())) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "车队油卡状态异常");
            }
        }
    }

    @Override
    public void delUserCard(String cardNo, Long userId) {
        HighUserCard card = getDetailByUserCardNo(userId, cardNo);
        if (card == null) {
            throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "未找到数据");
        }
        card.setStatus(0);
        editCard(card);

        if (card.getType().equals(UserCardType.type2.getType())) {
            // 查询卡号
            HighOilCard oilCard = oilCardService.getOilCardByCardNo(cardNo);
            if (oilCard == null) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.COMMON_ERROR, "油卡卡号错误");
            }
            oilCard.setBindTime(null);
            oilCard.setBindStatus(0);
            oilCardService.editData(oilCard);
        }
    }

    @Override
    public HighUserCard getDetailById(Long id) {
        return highUserCardMapper.selectByPrimaryKey(id);
    }

    @Override
    public List<HighUserCard> getListByUser(Long userId, Integer type) {
        HighUserCardExample example = new HighUserCardExample();
        HighUserCardExample.Criteria criteria = example.createCriteria().andUserIdEqualTo(userId).andStatusNotEqualTo(0);
        if (type != null) {
            criteria.andTypeEqualTo(type);
        }
        example.setOrderByClause("create_time desc");
        return highUserCardMapper.selectByExample(example);
    }

    @Override
    public HighUserCard getDetailByUserCardNo(Long userId, String cardNo) {
        HighUserCardExample example = new HighUserCardExample();
        example.createCriteria().andUserIdEqualTo(userId).andCardNoEqualTo(cardNo).andStatusNotEqualTo(0);
        List<HighUserCard> list = highUserCardMapper.selectByExample(example);
        if (list.size() > 0) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public List<HighUserCard> getDetailByCardNo(String cardNo) {
        HighUserCardExample example = new HighUserCardExample();
        example.createCriteria().andCardNoEqualTo(cardNo).andStatusNotEqualTo(0);
        return highUserCardMapper.selectByExample(example);
    }

    @Override
    public Boolean isBindHtlCard(Long userId) {
        HighUserCardExample example = new HighUserCardExample();
        example.createCriteria().andUserIdEqualTo(userId).andTypeEqualTo(UserCardType.type1.getType()).andStatusNotEqualTo(0);
        List<HighUserCard> list = highUserCardMapper.selectByExample(example);
        if (list.size() > 0) {
            return true;
        }
        return false;
    }

    @Override
    public HighUserCard getUserHtlCardNo(Long userId) {
        HighUserCardExample example = new HighUserCardExample();
        example.createCriteria().andUserIdEqualTo(userId).andTypeEqualTo(UserCardType.type1.getType()).andStatusNotEqualTo(0);
        List<HighUserCard> list = highUserCardMapper.selectByExample(example);
        if(list.size() > 0) {
            return list.get(0);
        }
        return null;
    }

    @Override
    public JSONObject getBlxCouponBalance(String phone) {

        try {
            if (phone == null) {
                return null;
            }

            JSONObject data = new JSONObject();
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("userMobile" , phone);
            jsonObject.put("couponStatus" , 1);

            HighUser user = highUserService.findByPhone(phone);

            // 获取比邻星接口查询用户停车券内容
            JSONObject object = blxConfig.getUserCouponPage(jsonObject);
            jsonObject.put("couponStatus" , 2);
            JSONObject object02 = blxConfig.getUserCouponPage(jsonObject);


            int num = 0;
            int num02 = 0;

            if (object.getInteger("code") == 1) {
                num = object.getJSONObject("data").getInteger("total");
                num02 = object02.getJSONObject("data").getInteger("total");

                if (num > 0 || num02 > 2) {

                    // 获取停车券内容
                    JSONArray blcArray = object.getJSONObject("data").getJSONArray("records");
                    blcArray.addAll(object02.getJSONObject("data").getJSONArray("records"));
                    data.put("data" , blcArray);

                    // 更新停车券状态
                    for (Object o : blcArray) {
                        JSONObject blxData = (JSONObject) o;
                        // 查询是否存在
                        HighUserCoupon userCoupon = highUserCouponService.findUserCouponByCode(blxData.getString("userCouponCode"));

                        HighCouponCodeOther couponCodeOther = couponCodeOtherService.getDetailByCouNo(blxData.getString("userCouponCode"));

                        // 查询卡券
                        HighCoupon coupon = highCouponService.getCouponDetail(blxData.getString("couponCode"));

                        if (userCoupon != null) {
                            userCoupon.setStatus(blxData.getInteger("userCouponStatus"));
                            highUserCouponService.updateUserCoupon(userCoupon);
                        } else {
                            HighUserCoupon highUserCoupon = new HighUserCoupon();
                            highUserCoupon.setStatus(blxData.getInteger("userCouponStatus"));
                            highUserCoupon.setUserPhone(phone);
                            if (coupon != null ) {
                                highUserCoupon.setCouponId(coupon.getId());
                            }

                            if (user != null) {
                                highUserCoupon.setUserId(user.getId());
                            }
                            highUserCoupon.setCreateTime(new Date());
                            highUserCoupon.setQrCodeImg(blxData.getString("userCouponCode"));
                            highUserCoupon.setUseEndTime(blxData.getDate("expiresTime"));
                            highUserCoupon.setSource(1);
                            highUserCoupon.setSourceType(6);
                            highUserCouponService.insertUserCoupon(highUserCoupon);

                        }

                        if (couponCodeOther != null) {
                            couponCodeOther.setStatus(blxData.getInteger("userCouponStatus") == 1 ? 20:40);
                            couponCodeOtherService.update(couponCodeOther);
                        } else {
                            HighCouponCodeOther codeOther  = new HighCouponCodeOther();
                            codeOther.setStatus(blxData.getInteger("userCouponStatus") == 1 ? 20:40);
                            codeOther.setType(4);
                            codeOther.setActiveTime(blxData.getDate("useStartTime"));
                            codeOther.setCouNo(blxData.getString("userCouponCode"));
                            codeOther.setCouTypeCode(blxData.getString("couponCode"));
                            codeOther.setCreateTime(new Date());
                            codeOther.setValidStartDate(blxData.getDate("useStartTime"));
                            codeOther.setValidEndDate(blxData.getDate("expiresTime"));
                            couponCodeOtherService.insertCouponCodeOther(codeOther);
                        }
                    }

                }

            }

            data.put("num" , num);

            return data;
        } catch (Exception e) {
            return null;
        }

    }
}
